@@ -29,9 +29,6 @@ G_BEGIN_DECLS
 
 char *panel_xdg_icon_remove_extension (const char *icon);
 
-char *panel_xdg_icon_name_from_icon_path (const char *path,
-					  GdkScreen  *screen);
-
 G_END_DECLS
 
 #endif /* PANEL_XDG_H */
