@@ -49,23 +49,13 @@ enum {
         PROP_DISABLE_LOCK_SCREEN,
         PROP_DISABLE_LOG_OUT,
         PROP_DISABLE_SWITCH_USER,
+        PROP_DISABLED_APPLETS,
         PROP_PANELS_LOCKED_DOWN,
         PROP_DISABLE_FORCE_QUIT
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (PanelLockdown, panel_lockdown, G_TYPE_OBJECT)
 
-static void
-_panel_lockdown_disabled_applets_changed (GSettings     *settings,
-                                          const char    *key,
-                                          PanelLockdown *lockdown)
-{
-        if (lockdown->priv->disabled_applets)
-                g_strfreev (lockdown->priv->disabled_applets);
-        lockdown->priv->disabled_applets = g_settings_get_strv (lockdown->priv->panel_settings,
-                                                                PANEL_LOCKDOWN_DISABLED_APPLETS_KEY);
-}
-
 static GObject *
 panel_lockdown_constructor (GType                  type,
                             guint                  n_construct_properties,
@@ -107,6 +97,12 @@ panel_lockdown_constructor (GType                  type,
                          "disable-switch-user",
                          G_SETTINGS_BIND_GET);
 
+        g_settings_bind (lockdown->priv->panel_settings,
+                         PANEL_LOCKDOWN_DISABLED_APPLETS_KEY,
+                         lockdown,
+                         "disabled-applets",
+                         G_SETTINGS_BIND_GET);
+
         g_settings_bind (lockdown->priv->panel_settings,
                          PANEL_LOCKDOWN_COMPLETE_LOCKDOWN_KEY,
                          lockdown,
@@ -119,15 +115,6 @@ panel_lockdown_constructor (GType                  type,
                          "disable-force-quit",
                          G_SETTINGS_BIND_GET);
 
-        g_signal_connect (lockdown->priv->panel_settings,
-                          "changed::"PANEL_LOCKDOWN_DISABLED_APPLETS_KEY,
-                          G_CALLBACK (_panel_lockdown_disabled_applets_changed),
-                          lockdown);
-
-        _panel_lockdown_disabled_applets_changed (lockdown->priv->panel_settings,
-                                                  PANEL_LOCKDOWN_DISABLED_APPLETS_KEY,
-                                                  lockdown);
-
         return obj;
 }
 
@@ -184,6 +171,10 @@ panel_lockdown_set_property (GObject      *object,
                                                      value,
                                                      "disable-switch-user");
                 break;
+        case PROP_DISABLED_APPLETS:
+                g_strfreev (lockdown->priv->disabled_applets);
+                lockdown->priv->disabled_applets = g_value_dup_boxed (value);
+                break;
         case PROP_PANELS_LOCKED_DOWN:
                 _panel_lockdown_set_property_helper (lockdown,
                                                      &lockdown->priv->panels_locked_down,
@@ -227,6 +218,9 @@ panel_lockdown_get_property (GObject    *object,
         case PROP_DISABLE_SWITCH_USER:
                 g_value_set_boolean (value, lockdown->priv->disable_switch_user);
                 break;
+        case PROP_DISABLED_APPLETS:
+                g_value_set_boxed (value, lockdown->priv->disabled_applets);
+                break;
         case PROP_PANELS_LOCKED_DOWN:
                 g_value_set_boolean (value, lockdown->priv->panels_locked_down);
                 break;
@@ -319,6 +313,16 @@ panel_lockdown_class_init (PanelLockdownClass *lockdown_class)
                         TRUE,
                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+        g_object_class_install_property (
+                gobject_class,
+                PROP_DISABLED_APPLETS,
+                g_param_spec_boxed (
+                        "disabled-applets",
+                        "Disabled applets",
+                        "Applet IIDs to disable from loading",
+                        G_TYPE_STRV,
+                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
         g_object_class_install_property (
                 gobject_class,
                 PROP_PANELS_LOCKED_DOWN,
@@ -363,30 +367,6 @@ panel_lockdown_get_disable_command_line (PanelLockdown *lockdown)
         return lockdown->priv->disable_command_line;
 }
 
-gboolean
-panel_lockdown_get_disable_lock_screen (PanelLockdown *lockdown)
-{
-        g_return_val_if_fail (PANEL_IS_LOCKDOWN (lockdown), TRUE);
-
-        return lockdown->priv->disable_lock_screen;
-}
-
-gboolean
-panel_lockdown_get_disable_log_out (PanelLockdown *lockdown)
-{
-        g_return_val_if_fail (PANEL_IS_LOCKDOWN (lockdown), TRUE);
-
-        return lockdown->priv->disable_log_out;
-}
-
-gboolean
-panel_lockdown_get_disable_switch_user (PanelLockdown *lockdown)
-{
-        g_return_val_if_fail (PANEL_IS_LOCKDOWN (lockdown), TRUE);
-
-        return lockdown->priv->disable_switch_user;
-}
-
 gboolean
 panel_lockdown_get_panels_locked_down (PanelLockdown *lockdown)
 {
@@ -395,14 +375,6 @@ panel_lockdown_get_panels_locked_down (PanelLockdown *lockdown)
         return lockdown->priv->panels_locked_down;
 }
 
-gboolean
-panel_lockdown_get_disable_force_quit (PanelLockdown *lockdown)
-{
-        g_return_val_if_fail (PANEL_IS_LOCKDOWN (lockdown), TRUE);
-
-        return lockdown->priv->disable_force_quit;
-}
-
 typedef struct {
         PanelLockdown       *lockdown;
         PanelLockdownNotify  callback;
@@ -498,38 +470,53 @@ panel_lockdown_get_disable_command_line_s (void)
         return panel_lockdown_get_disable_command_line (panel_lockdown_get ());
 }
 
-gboolean
-panel_lockdown_get_disable_lock_screen_s (void)
-{
-        return panel_lockdown_get_disable_lock_screen (panel_lockdown_get ());
-}
-
-gboolean
-panel_lockdown_get_disable_log_out_s (void)
-{
-        return panel_lockdown_get_disable_log_out (panel_lockdown_get ());
-}
-
-gboolean
-panel_lockdown_get_disable_switch_user_s (void)
-{
-        return panel_lockdown_get_disable_switch_user (panel_lockdown_get ());
-}
-
 gboolean
 panel_lockdown_get_panels_locked_down_s (void)
 {
         return panel_lockdown_get_panels_locked_down (panel_lockdown_get ());
 }
 
-gboolean
-panel_lockdown_get_not_panels_locked_down_s (void)
+GpLockdownFlags
+panel_lockdown_get_flags (PanelLockdown *lockdown,
+                          const char    *iid)
 {
-        return !panel_lockdown_get_panels_locked_down (panel_lockdown_get ());
+        GpLockdownFlags flags;
+        int i;
+
+        g_return_val_if_fail (PANEL_IS_LOCKDOWN (lockdown), GP_LOCKDOWN_FLAGS_NONE);
+
+        flags = GP_LOCKDOWN_FLAGS_NONE;
+
+        for (i = 0; lockdown->priv->disabled_applets[i] != NULL; i++) {
+                if (g_strcmp0 (lockdown->priv->disabled_applets[i], iid) == 0) {
+                        flags |= GP_LOCKDOWN_FLAGS_APPLET;
+                        break;
+                }
+        }
+
+        if (lockdown->priv->disable_force_quit)
+                flags |= GP_LOCKDOWN_FLAGS_FORCE_QUIT;
+
+        if (lockdown->priv->panels_locked_down)
+                flags |= GP_LOCKDOWN_FLAGS_LOCKED_DOWN;
+
+        if (lockdown->priv->disable_command_line)
+                flags |= GP_LOCKDOWN_FLAGS_COMMAND_LINE;
+
+        if (lockdown->priv->disable_lock_screen)
+                flags |= GP_LOCKDOWN_FLAGS_LOCK_SCREEN;
+
+        if (lockdown->priv->disable_log_out)
+                flags |= GP_LOCKDOWN_FLAGS_LOG_OUT;
+
+        if (lockdown->priv->disable_switch_user)
+                flags |= GP_LOCKDOWN_FLAGS_USER_SWITCHING;
+
+        return flags;
 }
 
-gboolean
-panel_lockdown_get_disable_force_quit_s (void)
+GpLockdownFlags
+panel_lockdown_get_flags_s (const char *iid)
 {
-        return panel_lockdown_get_disable_force_quit (panel_lockdown_get ());
+        return panel_lockdown_get_flags (panel_lockdown_get (), iid);
 }
